"use strict";
var DeclarationInjector = (function () {
    function DeclarationInjector(_a) {
        var common = _a.common, methods = _a.methods, properties = _a.properties;
        this._common = common;
        this._properties = properties;
        this._methods = methods;
    }
    Object.defineProperty(DeclarationInjector.prototype, "hasCommon", {
        get: function () {
            return !!this._common;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeclarationInjector.prototype, "hasMethods", {
        get: function () {
            return !!this._methods;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeclarationInjector.prototype, "hasProperties", {
        get: function () {
            return !!this._properties;
        },
        enumerable: true,
        configurable: true
    });
    DeclarationInjector.prototype.injectCommon = function (declaration) {
        declaration.$inject = this._common;
    };
    DeclarationInjector.prototype.injectMethods = function (declaration, methodName) {
        declaration[methodName].$inject = this._methods[methodName];
    };
    DeclarationInjector.prototype.injectProperties = function (declaration, ngModule) {
        var _this = this;
        ngModule.run(['$injector', function ($injector) {
                for (var property in _this._properties) {
                    var service = $injector.get(_this._properties[property]);
                    Reflect.defineMetadata('ngms:inject:property:get', service, declaration, property);
                }
            }]);
    };
    return DeclarationInjector;
}());
exports.DeclarationInjector = DeclarationInjector;
function bootstrapInject(declaration) {
    var injections = {};
    if (declaration.prototype && Reflect.hasMetadata('ngms:inject', declaration.prototype)) {
        injections.common
            = initCommon(Reflect.getMetadata('ngms:inject', declaration.prototype));
    }
    if (Reflect.hasMetadata('ngms:inject:method', declaration)) {
        injections.methods
            = initMethods(Reflect.getMetadata('ngms:inject:method', declaration));
    }
    if (Reflect.hasMetadata('ngms:inject:property', declaration.prototype)) {
        injections.properties
            = initProperties(Reflect.getMetadata('ngms:inject:property', declaration.prototype));
    }
    return new DeclarationInjector(injections);
}
exports.bootstrapInject = bootstrapInject;
function initCommon(data) {
    var common = new Array(data.length);
    for (var i = 0, len = data.length; i < len; i++) {
        var inject = data[i];
        common[i] = typeof inject === 'string' ? inject : inject.name;
    }
    return common;
}
function initMethods(data) {
    var methods = {};
    for (var method in data) {
        methods[method] = new Array(data[method].length);
        for (var i = 0, len = data[method].length; i < len; i++) {
            var inject = data[method][i];
            methods[method][i] = typeof inject === 'string' ? inject : inject.name;
        }
    }
    return methods;
}
function initProperties(data) {
    var properties = {};
    for (var property in data) {
        properties[property] = typeof data[property] === 'string'
            ? data[property]
            : data[property].name;
    }
    return properties;
}
//# sourceMappingURL=D:/Creation/1.Programming/javascript/ng-metasys/src/extensions/bootstrap-inject.js.map