"use strict";
function Inject() {
    var injections = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        injections[_i - 0] = arguments[_i];
    }
    return function (target, property, decorator) {
        if (property && target[property] && typeof target !== 'function') {
            throw new Error("Method for injection in declaration " + target.name + " should be static");
        }
        if (property && target[property]) {
            MethodInject(injections, target, property);
        }
        else if (property && !target[property]) {
            PropertyInject(injections, target, property, decorator);
        }
        else {
            CommonInject(injections, target);
        }
    };
}
exports.Inject = Inject;
function CommonInject(injections, target) {
    Reflect.defineMetadata('ngms:inject', injections, target.prototype);
}
function MethodInject(injections, target, property) {
    if (!Reflect.hasMetadata('ngms:inject:method', target)) {
        Reflect.defineMetadata('ngms:inject:method', (_a = {}, _a[property] = injections, _a), target);
        return;
    }
    Reflect.getMetadata('ngms:inject:method', target)[property] = injections;
    var _a;
}
function PropertyInject(injections, target, property, decorator) {
    if (injections.length > 1) {
        throw new Error('Only one injection can be added to a property');
    }
    if (decorator) {
        delete decorator.writable;
        delete decorator.initializer;
        decorator.configurable = false;
        decorator.get = function () { return Reflect.getMetadata('ngms:inject:property:get', target, property); };
    }
    else {
        Object.defineProperty(target, property, {
            configurable: false,
            enumerable: true,
            get: function () { return Reflect.getMetadata('ngms:inject:property:get', target, property); }
        });
    }
    var injection = injections[0];
    if (!Reflect.hasMetadata('ngms:inject:property', target)) {
        Reflect.defineMetadata('ngms:inject:property', (_a = {}, _a[property] = injection, _a), target);
        return;
    }
    Reflect.getMetadata('ngms:inject:property', target)[property] = injection;
    var _a;
}
//# sourceMappingURL=D:/Creation/1.Programming/javascript/ng-metasys/src/extensions/inject-decorator.js.map