"use strict";
var angular = require('angular');
var core_1 = require('../core');
var bootstrap_1 = require('../component/bootstrap');
var bootstrap_2 = require('../directive/bootstrap');
var bootstrap_3 = require('../filter/bootstrap');
var bootstrap_4 = require('../providers/bootstrap');
var bootstrap_module_config_1 = require('./bootstrap-module-config');
var moduleConfigTypes = ['value', 'constant', 'config', 'run'];
function bootstrapModule(declaration) {
    if (!Reflect.hasMetadata('ngms:module', declaration.prototype)) {
        throw new Error(declaration.name + " is not marked with @Module decorator");
    }
    var metadata = Reflect.getMetadata('ngms:module', declaration.prototype);
    var imports = metadata.imports ? initImports(metadata.imports) : [];
    var ngModule = angular.module(declaration.name, imports);
    core_1.NgmsReflect.modules.set(declaration.name, ngModule);
    if (metadata.declarations) {
        initDeclarations(ngModule, metadata.declarations);
    }
    initConfig(ngModule, declaration);
    if (metadata.providers) {
        for (var _i = 0, _a = metadata.providers; _i < _a.length; _i++) {
            var provider = _a[_i];
            bootstrap_4.bootstrapProviders(ngModule, provider);
        }
    }
    return ngModule.name;
}
exports.bootstrapModule = bootstrapModule;
function initImports(imports) {
    var i = 0;
    var modules = new Array(imports.length);
    for (var _i = 0, imports_1 = imports; _i < imports_1.length; _i++) {
        var importing = imports_1[_i];
        modules[i] = typeof importing === 'string' ? importing : bootstrapModule(importing);
        i += 1;
    }
    return modules;
}
function initDeclarations(ngModule, declarations) {
    for (var _i = 0, declarations_1 = declarations; _i < declarations_1.length; _i++) {
        var declaration = declarations_1[_i];
        switch (true) {
            case Reflect.hasMetadata('ngms:component', declaration.prototype):
                bootstrap_1.bootstrapComponent(ngModule, declaration);
                break;
            case Reflect.hasMetadata('ngms:directive', declaration.prototype):
                bootstrap_2.bootstrapDirective(ngModule, declaration);
                break;
            case Reflect.hasMetadata('ngms:filter', declaration.prototype):
                bootstrap_3.bootstrapFilter(ngModule, declaration);
                break;
            default:
                throw new Error("Unknown declaration " + declaration.name + " in module " + ngModule.name);
        }
    }
}
function initConfig(ngModule, declaration) {
    for (var _i = 0, moduleConfigTypes_1 = moduleConfigTypes; _i < moduleConfigTypes_1.length; _i++) {
        var type = moduleConfigTypes_1[_i];
        if (!Reflect.hasMetadata("ngms:module:" + type, declaration)) {
            continue;
        }
        var properties = bootstrap_module_config_1.bootstrapModuleConfig(declaration, type);
        for (var _a = 0, properties_1 = properties; _a < properties_1.length; _a++) {
            var property = properties_1[_a];
            switch (type) {
                case 'config':
                case 'run':
                    ngModule[type](declaration[property]);
                    break;
                default:
                    ngModule[type](property, declaration[property]);
                    break;
            }
        }
    }
}
//# sourceMappingURL=D:/Creation/1.Programming/javascript/ng-metasys/src/module/bootstrap-module.js.map